from typing import Union


def get_bytes(file: Union[str, bytes]) -> bytes:
    if isinstance(file, str):
        with open(file, "rb") as fh:
            file_bytes = fh.read()
    elif isinstance(file, bytes):
        file_bytes = file
    else:
        raise TypeError
    return file_bytes


def validate_header(data: bytes, header: bytes, offset: int = 0) -> bool:
    return data[offset : len(header) + offset] == header


def validate_trailer(data: bytes, trailer: bytes, offset: int = 0) -> bool:
    length = len(data)
    return data[length - len(trailer) - offset : length - offset] == trailer



from typing import Union

from filext.utils import get_bytes, validate_header, validate_trailer


def is_microsoft_openxml(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\x50\x4B\x03\x04\x14\x00\x06\x00"
    header_offset = 0
    trailer = b"\x50\x4B\x05\x06"
    trailer_offset = 18

    return validate_header(data, header, header_offset) and validate_trailer(
        data, trailer, trailer_offset
    )


def is_docx(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    return is_microsoft_openxml(data) and b"word/document.xml" in data


def is_xlsx(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    return is_microsoft_openxml(data) and b"xl/workbook.xml" in data


def is_pptx(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    return is_microsoft_openxml(data) and b"ppt/presentation.xml" in data


def is_pdf(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\x25\x50\x44\x46"
    return validate_header(data, header)


def is_png(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\x89\x50\x4E\x47\x0D\x0A\x1A\x0A"
    trailer = b"\x49\x45\x4E\x44\xAE\x42\x60\x82"
    return validate_header(data, header) and validate_trailer(data, trailer)


def is_jpg(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\xFF\xD8\xFF"
    trailer = b"\xFF\xD9"
    return validate_header(data, header) and validate_trailer(data, trailer)


def is_gif(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header1 = b"\x47\x49\x46\x38\x37\x61"
    header2 = b"\x47\x49\x46\x38\x39\x61"
    return validate_header(data, header1) or validate_header(data, header2)


def is_bmp(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\x42\x4D"
    return validate_header(data, header)


def is_tif(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"\x49\x49\x2A\x00"
    return validate_header(data, header)


def is_heic(file: Union[str, bytes]) -> bool:
    data = get_bytes(file)
    header = b"ftypheic"
    offset = 4
    return validate_header(data, header, offset)



import pytest

from filext.utils import get_bytes


@pytest.mark.parametrize("file", ["tests/files/image.png"])
def test_get_bytes_valid(file):
    assert isinstance(get_bytes(file), bytes)


@pytest.mark.parametrize(
    "file",
    [
        0,
        None,
        [],
        {},
        int,
        len,
    ],
)
def test_get_bytes_invalid(file):
    with pytest.raises(TypeError):
        get_bytes(file)



import pytest

from filext import checkers


@pytest.mark.parametrize(
    "filepath, checker",
    [
        ("tests/files/document.docx", checkers.is_docx),
        ("tests/files/document.pptx", checkers.is_pptx),
        ("tests/files/document.xlsx", checkers.is_xlsx),
        ("tests/files/document.pdf", checkers.is_pdf),
        ("tests/files/document.pdf", checkers.is_pdf),
    ],
)
def test_document_checkers_valid(filepath, checker):
    assert checker(filepath)


@pytest.mark.parametrize(
    "filepath, checker",
    [
        ("tests/files/document.docx", checkers.is_pptx),
        ("tests/files/document.pptx", checkers.is_xlsx),
        ("tests/files/document.xlsx", checkers.is_pdf),
        ("tests/files/document.pdf", checkers.is_docx),
    ],
)
def test_document_checkers_invalid(filepath, checker):
    assert not checker(filepath)


@pytest.mark.parametrize(
    "filepath, checker",
    [
        ("tests/files/image.bmp", checkers.is_bmp),
        ("tests/files/image.gif", checkers.is_gif),
        ("tests/files/image.heic", checkers.is_heic),
        ("tests/files/image.jpg", checkers.is_jpg),
        ("tests/files/image.png", checkers.is_png),
        ("tests/files/image.tif", checkers.is_tif),
    ],
)
def test_image_checkers_valid(filepath, checker):
    assert checker(filepath)



import io
import os

import setuptools

# Import the README and use it as the long-description.
try:
    with io.open(
        os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"),
        encoding="utf-8",
    ) as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = ""


setuptools.setup(
    name="filext",
    version="0.1.1",
    description="Python library to identify file type based on its file signature",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dobizz/filext",
    author="Ronnie Villanueva",
    author_email="ronnie.code@outlook.com",
    packages=setuptools.find_packages(),
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
